@@ -20,12 +20,14 @@
  */
 
 #include "ephy-action-bar-end.h"
-#include "ephy-add-bookmark-popover.h"
+#include "ephy-bookmarks-dialog.h"
+#include "ephy-bookmark-properties.h"
 #include "ephy-browser-action.h"
 #include "ephy-browser-action-row.h"
 #include "ephy-desktop-utils.h"
 #include "ephy-downloads-paintable.h"
 #include "ephy-downloads-popover.h"
+#include "ephy-embed-container.h"
 #include "ephy-shell.h"
 #include "ephy-web-extension-manager.h"
 #include "ephy-window.h"
@@ -214,6 +216,25 @@ browser_action_popover_visible_changed_cb (GtkWidget        *popover,
   }
 }
 
+static void
+on_bookmarks_button (GtkButton *button,
+                     gpointer   user_data)
+{
+  GtkWindow *parent_window = GTK_WINDOW (gtk_widget_get_root (GTK_WIDGET (button)));
+  ephy_window_toggle_bookmarks (EPHY_WINDOW (parent_window));
+}
+
+static void
+on_bookmark_button_clicked (GtkButton *button,
+                            gpointer   user_data)
+{
+  GtkWidget *window = GTK_WIDGET (gtk_widget_get_root (GTK_WIDGET (button)));
+  GtkWidget *dialog;
+
+  dialog = ephy_bookmark_properties_new_for_window (EPHY_WINDOW (window));
+  adw_dialog_present (ADW_DIALOG (dialog), window);
+}
+
 static void
 ephy_action_bar_end_class_init (EphyActionBarEndClass *klass)
 {
@@ -266,6 +287,8 @@ ephy_action_bar_end_class_init (EphyActionBarEndClass *klass)
                                            browser_actions_popup_view_back_clicked_cb);
   gtk_widget_class_bind_template_callback (widget_class,
                                            browser_actions_row_activated_cb);
+  gtk_widget_class_bind_template_callback (widget_class,
+                                           on_bookmarks_button);
 }
 
 static void
@@ -273,7 +296,6 @@ ephy_action_bar_end_init (EphyActionBarEnd *action_bar_end)
 {
   GObject *object = G_OBJECT (action_bar_end);
   EphyDownloadsManager *downloads_manager;
-  GtkWidget *popover;
   EphyWebExtensionManager *extension_manager;
   EphyEmbedShell *embed_shell;
 
@@ -296,8 +318,8 @@ ephy_action_bar_end_init (EphyActionBarEnd *action_bar_end)
                                 action_bar_end->downloads_paintable);
 
   if (is_desktop_pantheon ()) {
-    gtk_menu_button_set_icon_name (GTK_MENU_BUTTON (action_bar_end->bookmarks_button),
-                                   "user-bookmarks");
+    gtk_button_set_icon_name (GTK_BUTTON (action_bar_end->bookmarks_button),
+                              "user-bookmarks");
     gtk_menu_button_set_icon_name (GTK_MENU_BUTTON (action_bar_end->overview_button),
                                    "view-grid");
   }
@@ -321,9 +343,7 @@ ephy_action_bar_end_init (EphyActionBarEnd *action_bar_end)
                            G_CALLBACK (show_downloads_cb),
                            object, 0);
 
-  popover = ephy_add_bookmark_popover_new ();
-
-  gtk_menu_button_set_popover (GTK_MENU_BUTTON (action_bar_end->bookmark_button), popover);
+  g_signal_connect_object (action_bar_end->bookmark_button, "clicked", G_CALLBACK (on_bookmark_button_clicked), action_bar_end, 0);
 
   extension_manager = ephy_web_extension_manager_get_default ();
   g_signal_connect_object (extension_manager, "show-browser-action",
@@ -358,19 +378,6 @@ ephy_action_bar_end_show_downloads (EphyActionBarEnd *action_bar_end)
     gtk_menu_button_popup (GTK_MENU_BUTTON (action_bar_end->downloads_button));
 }
 
-void
-ephy_action_bar_end_show_bookmarks (EphyActionBarEnd *action_bar_end)
-{
-  if (gtk_widget_get_visible (action_bar_end->bookmarks_button))
-    gtk_menu_button_popup (GTK_MENU_BUTTON (action_bar_end->bookmarks_button));
-}
-
-GtkWidget *
-ephy_action_bar_end_get_bookmarks_button (EphyActionBarEnd *action_bar_end)
-{
-  return action_bar_end->bookmarks_button;
-}
-
 GtkWidget *
 ephy_action_bar_end_get_downloads_revealer (EphyActionBarEnd *action_bar_end)
 {
@@ -389,15 +396,15 @@ ephy_action_bar_end_set_bookmark_icon_state (EphyActionBarEnd      *action_bar_e
       break;
     case EPHY_BOOKMARK_ICON_EMPTY:
       gtk_widget_set_visible (action_bar_end->bookmark_button, TRUE);
-      gtk_menu_button_set_icon_name (GTK_MENU_BUTTON (action_bar_end->bookmark_button),
-                                     "ephy-non-starred-symbolic");
+      gtk_button_set_icon_name (GTK_BUTTON (action_bar_end->bookmark_button),
+                                "ephy-non-starred-symbolic");
       /* Translators: tooltip for the empty bookmark button */
       gtk_widget_set_tooltip_text (action_bar_end->bookmark_button, _("Bookmark Page"));
       break;
     case EPHY_BOOKMARK_ICON_BOOKMARKED:
       gtk_widget_set_visible (action_bar_end->bookmark_button, TRUE);
-      gtk_menu_button_set_icon_name (GTK_MENU_BUTTON (action_bar_end->bookmark_button),
-                                     "ephy-starred-symbolic");
+      gtk_button_set_icon_name (GTK_BUTTON (action_bar_end->bookmark_button),
+                                "ephy-starred-symbolic");
 
       /* Translators: tooltip for the bookmarked button */
       gtk_widget_set_tooltip_text (action_bar_end->bookmark_button, _("Edit Bookmark"));
